/*
	Xiliinx ISP parallel port I/F Driver DLL Version 1.0

	Copyright(c) 2002,2003 Tatsuyuki Satoh

	XilinxpP[uɓKp
	^[Qbgubb̃`FbN@\
	I/O𒼐ڂ̂߁AWindowsNT/200/XPłgiveio.sysKv

	PDoE_XL(JTAG)[h}

	D0    -> TDI
	D1    -> TCK
	D2    -> TMS
	D3    -> -ENABLE TDI,TCK,TMS
	D4    -> -ENABLE TDO=L (for VCC sense)
	D6    -> PROBLE SENSE(to PE & BUSY)
	SLCT  <- TDI
	ERROR <- TargetVCC
	BUSY  <- PROBLE SENSE(from D6)
	PE    <- PROBLE SENSE(from D6)

	QDVAX[u[h}

	D0    -> DIN
	D1    -> CCLK
	D2    -> PROGRAM
	D3    -> -ENABLE DIN,CCLK,PROGARM
	D4    -> -ENABLE DONE=L (for VCC sense)
	D6    -> PROBLE SENSE(to PE & BUSY)
	SLCT  <- DONE
	ERROR <- TargetVCC
	BUSY  <- PROBLE SENSE(from D6)
	PE    <- PROBLE SENSE(from D6)

	Export DLL Functions

	JTAG BounteryScan
		int jtag_version(void)
		int jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
		int jtag_power(void)
		int jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
		void jtag_end(void)

	CCLK SerialSlave
		int ss_version(void)
		int ss_init(DWORD port_address,DWORD baudrate,char *probe_name);
		int ss_power(void);
		int ss_tx(const BYTE *data,DWORD size);
		int ss_program(BYTE level);
		int ss_done(void);
		void ss_end(void);

	note:

		RpC̓t[̃RpCAMINGWgpĂ܂B

	Histry
		2001. 3.18 1st release
		2002. 7.29 DLL conversion
		           use GIVEIO for WindowsNT/2000/XP
		2003. 1.10 support GIVE I/O auto Installall/Remove

*/
#include <windows.h>
#include <stdio.h>
#include "giveio.h"

// DLL Export prefix
#define DLLEXPORT __declspec(dllexport)
#ifdef __MINGW32__
#define DLLAPI
#else
#define DLLAPI WINAPI
#endif

/* ---------------------------------------------------------- */

//ph^egptO@^@h^nAhX
static DWORD par_port;

//giveio
static int giveio_result;

// JTAG out 1clock
#define PAR_JTAG(TMS,TDI) {\
	_outp(par_port,(TMS?4:0) | (TDI?1:0) | 0x10); \
	_outp(par_port,(TMS?4:0) | (TDI?1:0) | 0x12); \
}

// JTAG TDO sense
#define PAR_TDO() ((_inp(par_port+1)>>4)&1)

// VCC sense
#define PAR_VCC() ((_inp(par_port+1)>>3)&1)
// probe sense
#define PAR_PR0() ((_inp(par_port+1)>>5)&1)
#define PAR_PR1() ((_inp(par_port+1)>>7)&1)

DLLEXPORT DWORD DLLAPI jtag_version(void)
{
	DWORD mjr = 1;
	DWORD mnr = 0;
	return (mjr<<16) + mnr;
}

//p|[g̏
DLLEXPORT int DLLAPI jtag_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	par_port = port_address==0 ? 0x378 : port_address;

	if( probe_name != NULL)
		sprintf(probe_name,"Xilinx Parallel ISP Cable (JTAG),IO Base=%04X",(int)port_address);

	// GIVEIO Service
	giveio_result = giveio_start();
	if(giveio_result==GIVEIO_ERROR)
	{
		printf("Can't access GIVEIO system\n");
		return 1;
	}

	// probe sense
	_outp(par_port,0x50);
	if( !PAR_PR0() || PAR_PR1() )
	{
		printf("No Probe\n");
		return 1;
	}
	_outp(par_port,0x10);
	if( PAR_PR0() || !PAR_PR1() )
	{
		printf("No Probe\n");
		return 1;
	}

	return 0;
}

//d`FbN
DLLEXPORT int DLLAPI jtag_power(void)
{
	return  PAR_VCC() ? 1: 0;
}

//is`fɂ鑗M
DLLEXPORT int DLLAPI jtag_txrx(const BYTE *ibuf,BYTE *obuf,DWORD drsize,int tms_s,int tms_e)
{
	int c;

	// clear outbuf
	if(obuf)
		for(c=0;c<(drsize+7)/8;c++)
			obuf[c] = 0;

	for(c=0;c<drsize;c++)
	{
		int tdi = ibuf && (ibuf[c/8]&(0x80>>(c&7))) ? 1 : 0;
		int tms = (c==drsize-1) ? tms_e : tms_s;
		// out 1 TCK
		PAR_JTAG(tms,tdi);
		// TDO get
		if( obuf && PAR_TDO() )
			obuf[c/8] |= 0x01<<(c&7);
	}
	return 0;
}

DLLEXPORT void DLLAPI jtag_end(void)
{
	// Release & Remove GIVEIO
	giveio_result = giveio_stop(giveio_result);
}

/*-------------------------------------------------------------------------------*/
#define CF_DIN  0x01
#define CF_CCLK 0x02
#define CF_PROGRAM 0x04
#define CF_BASE (0x10|CF_PROGRAM)

#define PAR_DONE() ((_inp(par_port+1)>>4)&1)

//return version
DLLEXPORT int DLLAPI ss_version(void)
{
	DWORD mjr = 1;
	DWORD mnr = 0;
	return (mjr<<16) + mnr;
}

DLLEXPORT int DLLAPI ss_init(DWORD port_address,DWORD baudrate,char *probe_name)
{
	if( probe_name != NULL)
		sprintf(probe_name,"Xilinx Parallel ISP Cable (CCLK),IO Base=%04X",(int)port_address);

	return jtag_init(port_address,baudrate,NULL);
}

DLLEXPORT int DLLAPI ss_power(void)
{
	return jtag_power();
}

DLLEXPORT int DLLAPI ss_tx(const BYTE *data,DWORD size)
{
	int pos;

	// PubN̑Mf[^쐬
	for(pos=0 ; pos<size ; pos++)
	{
		BYTE value  = data[pos];

		BYTE outval;
		outval = (value&0x80) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x40) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x20) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x10) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x08) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x04) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x02) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
		outval = (value&0x01) ? CF_BASE | CF_DIN : CF_BASE;
		_outp(par_port,outval);
		_outp(par_port,outval|CF_CCLK);
	}
	return 0;


	return 1;
}

DLLEXPORT int DLLAPI ss_program(BYTE level)
{
	_outp(par_port,0x10);
	_outp(par_port,0x10);
	_outp(par_port,CF_BASE);
	return 0;
}

DLLEXPORT int DLLAPI ss_done(void)
{
	return PAR_DONE();
}

DLLEXPORT void DLLAPI ss_end(void)
{
}
